<?

/**
 * Hijri Shamsi Date Time Class
 * by: Michel Kogan
 * --------------------
 * LICENSE
 * --------------------
 * This program is open source product; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License (GPL)
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * To read the license please visit http://www.gnu.org/copyleft/gpl.html
 * 
 * --------------------
 * @package    HijriShamsiDateTime
 * @author     Michel Kogan <kogan.michel@gmail.com
 * @copyright  2012 Michel Kogan
 * @license    http://www.gnu.org/copyleft/gpl.html  GNU General Public License (GPL)
 * @link       http://www.sparta.ir
 * @see        DateTime
 * @version    1.0.1
 * 
 *  
 */
class HijriShamsiDateTime {
  
    /**
     * HijriShamsiDateTime::Constructor
     *
     * Pass these parameteres when creating a new instance
     * of this Class, and they will be used as defaults.
     * e.g $obj = new HijriShamsiDateTime("fa");
     * To use system defaults pass null for english names
     * @param $lang String language - "fa" for Persian(Farsi) months name
     */
    function __construct() {
    }

    public function init() {
    }
    
    /**
     * returns a string formatted according to the given format string using the given
     * integer timestamp or the current time if no timestamp is given. In other words, timestamp 
     * is optional and defaults to the value of time(). 
     * 
     * @param $format String date format (see http://php.net/manual/en/function.date.php)
     * @param $timestamp Integer time measured in the number of seconds since the Unix Epoch (January 1 1970 00:00:00 GMT)
     * 
     * @return String Returns the Hijri Shamsi date according to format and timestamp in Farsi/English
     *
     */
    function date($format, $timestamp=0) {
        if ($timestamp === 0) {
            $timestamp = time();
        }
        list ($d, $D, $j, $l, $S, $F, $m, $M, $n, $t, $L, $o, $Y, $y, $w, $a, $A, $H, $i, $s, $O) = explode("/", date("d/D/j/l/S/F/m/M/n/t/L/o/Y/y/w/a/A/H/i/s/O", $timestamp));
        extract($this->GregorianToHijriShamsi($d, $m, $Y));
        $j = $day;
        $t = 0;
        $L = 0;
        $d = sprintf("%02d", $day);
        $m = sprintf("%02d", $month);
        $n = $month;
        $Y = $year;
        $y = substr($year, 2);
        $S = substr($j, -1) == 1 ? "st" : (substr($j, -1) == 2 ? "nd" : (substr($j, -1) == 3 ? "rd" : "th"));
        $F = Yii::t('HijriShamsi', 'mon.' . $n);
        $M = Yii::t('HijriShamsi', 'mon.' . $n);


        $D = Yii::t('HijriShamsi', 'day.' . $w);
        $l = Yii::t('HijriShamsi', 'day.' . $w);
        $S = "";
        $a = Yii::t('HijriShamsi', 'tm.' . $a);
        $A = Yii::t('HijriShamsi', 'tm.' . $A);
        $r = "$D, $j $M $Y $H:$i:$s $O";
        $davars = array("d", "D", "j", "l", "S", "F", "m", "M", "n", "t", "L", "o", "Y", "y", "a", "A", "r");
        $myvars = array($d, "¢", $j, "£", "ç", "¥", $m, "©", $n, $t, $L, $Y, $Y, $y, "ï", "â", "®");
        $format = str_replace($davars, $myvars, $format);
        $date = date($format, $timestamp);
        $date = str_replace(array("¢", "£", "ç", "¥", "©", "ï", "â", "®"), array($D, $l, $S, $F, $M, $a, $A, $r), $date);
        return $date;
    }
    
    /**
     * Returns an array of  month, day, year 
     * 
     * @param $day Integer Gregorian day of the month
     * @param $month Integer Gregorian month number
     * @param $year Integer Gregorian year in full (1999)
     * @return Array Hijri Shamsi date[int month, int day, int year]
     */
    public function GregorianToHijriShamsi($day=20, $month=02, $year=1976) {
        $g_days_in_month = array(31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
        $j_days_in_month = array(31, 31, 31, 31, 31, 31, 30, 30, 30, 30, 30, 29);
    
        $gy = $year-1600;
        $gm = $month-1;
        $gd = $day-1;
    
        $g_day_no = 365*$gy+(int)(($gy+3)/4)-(int)(($gy+99)/100)+(int)(($gy+399)/400);
    
        for ($i=0; $i < $gm; ++$i)
            $g_day_no += $g_days_in_month[$i];
            
        if ($gm>1 && (($gy%4==0 && $gy%100!=0) || ($gy%400==0)))
            /* leap and after Feb */
            $g_day_no++;
            
        $g_day_no += $gd;
    
        $j_day_no = $g_day_no-79;
    
        $j_np = (int)($j_day_no/12053); /* 12053 = 365*33 + 32/4 */
        $j_day_no = $j_day_no % 12053;
    
        $year = 979+33*$j_np+4*(int)($j_day_no/1461); /* 1461 = 365*4 + 4/4 */
    
        $j_day_no %= 1461;
    
        if ($j_day_no >= 366) {
            $year += (int)(($j_day_no-1)/ 365);
            $j_day_no = ($j_day_no-1)%365;
        }
    
        for ($i = 0; $i < 11 && $j_day_no >= $j_days_in_month[$i]; ++$i)
            $j_day_no -= $j_days_in_month[$i];
        $month = $i+1;
        $day = $j_day_no+1;


        list ($_Date["month"], $_Date["day"], $_Date["year"]) = array($month, $day, $year);
        return ($_Date);
    }

    /**
     * Parse about any English textual datetime description into a Hijri Shamsi format
     * if no format the date returned will be Hijri Shamsi d-m-Y 
     * @param String $strDate English textual datetime
     * @param String $format The date format needed
     * @param String $sep separator
     * @return String Date in Hijri Shamsi as formated or like d-m-Y 
     */
    public function strToHijriShamsi($strDate, $format= "d-m-Y") {
        if ($format == "d-m-Y") {
            list ($d, $m, $Y) = explode("/", date("d/m/Y", strtotime($strDate)));
            extract($this->GregorianToHijriShamsi($d, $m, $Y));
            $date = "$day-$month-$year";
        } else {
            $date = $this->date($format, strtotime($strDate));
        }
        return $date;
    }
    
}

?>